/* global Configuration, ZhuAclMessages */
function ZhuAclCRUDController(ScreenService, ZHPromise, ZhuAclConfig, ZhuAclController, MessageService){

	function defineQuestionToUse(rule){
		var question = ZhuAclMessages.VISIBLE;
		if(rule == ZhuAclConfig.RULE_INVISIBLE){
			question = ZhuAclMessages.INVISIBLE;
		} else if(rule == ZhuAclConfig.RULE_ALLOW){
			question = ZhuAclMessages.EDITABLE;
		}
		return {newVisibility: ScreenService.i18n(question)};
	}

	function deleteChildrensRules(elementsWidget, parentRow, groupId, ruleToContainer){
		if(parentRow._childCount){
			elementsWidget.dataSource.data.forEach(function(row){
				if(row._parent == parentRow){
					row.changed = true;
					row.__isDeleted = true;
					row.GROUP_ID = groupId;
					row.RULE_SAVE_CONTAINER = ruleToContainer;
					deleteChildrensRules(elementsWidget, row, groupId, ruleToContainer);
				}
			});
		}
	}

	function confirmChildsVisibility(elementsWidget, ruleType){
		var returnPromise = ZHPromise.when();
		if(elementsWidget.currentRow._childCount){
			var currentRow = elementsWidget.currentRow;
			var defer = ZHPromise.defer();
			var actions = [
				{
					label: ScreenService.i18n(ZhuAclMessages.YES),
					code: function(){
						currentRow.changed = true;
						deleteChildrensRules(elementsWidget, currentRow, currentRow.GROUP_ID, currentRow.RULE);
						defer.resolve();
					}
				},
				{
					label: ScreenService.i18n(ZhuAclMessages.NO),
					code: function(){
						var messageConfig = {
							type: 'confirmation',
							icon: 'question'
						};
						MessageService.showMessageByCode(ZhuAclMessages.CONFIRMATION_MESSAGE, {}, messageConfig);
						defer.reject();
					}
				}
			];
			var dialogConfig = {
				type: 'confirmation',
				icon: 'question',
				actions: actions
			};

			MessageService.showCustomDialogByCode(ZhuAclMessages.CONFIRM_CHILDS_QUESTION, defineQuestionToUse(ruleType), dialogConfig);
			returnPromise = defer.promise;
		}

		return returnPromise;
	}

	function reloadChildrensRules(elementsWidget, parentRow, updatedRules, deletedRules){
		if(parentRow._childCount){
			elementsWidget.dataSource.data.forEach(function(row){
				if(row._parent == parentRow){
					var rowId = row.UNIQUE_ID;
					row.isVisible = parentRow.isVisible;
					row.isEditable = parentRow.isEditable;

					if(deletedRules.indexOf(rowId) != -1){
						delete row.RULE_ID;
					}
					if(updatedRules[rowId] && updatedRules[rowId].containerId){
						row.CONTAINER_RULE_ID = updatedRules[rowId].containerId;
					}

					reloadChildrensRules(elementsWidget, row, updatedRules, deletedRules);
				}
				delete row.__isDeleted;
				delete row.changed;
			});
		}
	}

	function fixZeedhiBehavior(widget){
		widget.dataSource.data.forEach(function(data){
			data.isEditable = (data._childCount && data.isVisible) || data.isEditable;
		});
	}

	function isFatherVisible(parentRow, widget){
		return widget.dataSource.data.some(function(row){
			return (row.PARENT_ID == parentRow.UNIQUE_ID) && (row.isVisible || row.isEditable);
		});
	}

	function reloadParentsRules(row, widget){
		var parentRow = row._parent;
		if(parentRow){
			if(row.isVisible){
				parentRow.isEditable = parentRow.isEditable || row.isEditable || row.isVisible;
				parentRow.isVisible = true;
			} else {
				var updateFather = isFatherVisible(parentRow, widget);
				parentRow.isEditable = updateFather;
				parentRow.isVisible = updateFather;
			}
			reloadParentsRules(row._parent, widget);
		}
	}

	function reloadDataSourceAfterSave(elementsWidget, newRules){
		var currentRow = elementsWidget.currentRow;
		var currentRowId = currentRow.UNIQUE_ID;
		var updatedRule = newRules.updated[currentRowId];
		var deletedRules = newRules.deleted || [];
		if(updatedRule){
			if(Util.isObject(updatedRule)){
				currentRow.RULE_ID = updatedRule.menuId;
				currentRow.CONTAINER_RULE_ID = updatedRule.containerId;
			} else {
				currentRow.RULE_ID = updatedRule;
			}
			reloadChildrensRules(elementsWidget, currentRow, newRules.updated, deletedRules);
			reloadParentsRules(currentRow, elementsWidget);
			fixZeedhiBehavior(elementsWidget);
		}
	}

	function cancelSave(widget){
		widget.reload();
	}

	function changeElementRule(currentRow, isVisible, isEditable){
		currentRow.RULE = ZhuAclConfig.getRuleType(isVisible, isEditable);
		currentRow.isVisible = isVisible;
		currentRow.isEditable = isEditable;
		currentRow.GROUP_ID = currentRow.GROUP_ID || ZhuAclController.getCurrentGroupId();
	}

	function clearInlineChangedStatus(row){
		delete row.changed_isEditable;
		delete row.changed_isVisible;
	}

	function saveRule(widget, isEditable, isVisible){
		var currentRow = widget.currentRow;
		var ruleType = ZhuAclConfig.getRuleType(isVisible, isEditable);

		clearInlineChangedStatus(currentRow);
		changeElementRule(currentRow, isVisible, isEditable);

		confirmChildsVisibility(widget, ruleType).then(function(){
			widget.saveRow().then(function(response){
				reloadDataSourceAfterSave(widget, response.dataset.aclZhuAccessControlRule);
				MessageService.successNotificationByCode(ZhuAclMessages.SUC_RULE_INSERTED);
			}).catch(function(){
				cancelSave(widget);
				MessageService.notificationMessageByCode(ZhuAclMessages.ERR_RULE_INSERTED, {}, {type: "alert"});
			});
		}).catch(function(){
			cancelSave(widget);
			MessageService.notificationMessageByCode(ZhuAclMessages.RULE_NOT_SAVED, {}, {type: "alert"});
		});
	}

	this.changeIsEditable = function changeIsEditable(widget, row){
		var isEditable = row.isEditable;
		var isVisible = isEditable || row.isVisible;
		saveRule(widget, isEditable, isVisible);
	};

	this.changeIsVisible = function changeIsVisible(widget, row){
		var isVisible = row.isVisible;
		var isEditable = isVisible && row.isEditable;
		saveRule(widget, isEditable, isVisible);
	};
}

Configuration(function(ContextRegister){
	ContextRegister.register('ZhuAclCRUDController', ZhuAclCRUDController);
});
